package com.persagy.proxy.adm.utils;

import com.fasterxml.jackson.databind.node.ObjectNode;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.locationtech.jts.geom.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @ClassName GeoToolsUtil
 * @Description: 点面包含关系分析
 * @Author linhuili
 * @Date 2021/9/3 20:31
 * @Version V1.0
 **/
public class GeoToolsUtil {
    /**
     * 创建点
     * @param pointLocal
     */
    private static Point createPoint(ObjectNode pointLocal){
        Long x = 0L;
        Long y = 0L;
        Long z = 0L;
        if(pointLocal.get("X") != null){
            x = Long.valueOf(pointLocal.get("X").textValue());
        }
        if(pointLocal.get("Y") != null){
            y = Long.valueOf(pointLocal.get("Y").textValue());
        }
        if(pointLocal.get("Z") !=null){
            z = Long.valueOf(pointLocal.get("Z").textValue());
        }
        GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory();
        Coordinate coord = new Coordinate(x, y,z);
        Point point = geometryFactory.createPoint(coord);
        return point;
    }

    /**
     * 创建面
     * @param outLines
     */
    private static Polygon createPolygon(List<ObjectNode> outLines){
        GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory();
        List<Coordinate> objects = new ArrayList<>();
        for (ObjectNode outLine : outLines) {
            Long x = 0L;
            Long y = 0L;
            Long z = 0L;
            if(outLine.get("X") != null){
                x = Long.valueOf(outLine.get("X").textValue());
            }
            if(outLine.get("Y") != null){
                y = Long.valueOf(outLine.get("Y").textValue());
            }
            if(outLine.get("Z") !=null){
                z = Long.valueOf(outLine.get("Z").textValue());
            }
            Coordinate coordinate = new Coordinate(x, y, z);
            objects.add(coordinate);
        }
        Coordinate[] coordinates = objects.toArray(new Coordinate[0]);
        LinearRing ring = geometryFactory.createLinearRing(objects.toArray(new Coordinate[0]));
        LinearRing holes[] = null; // use LinearRing[] to represent holes
        Polygon polygon = geometryFactory.createPolygon(ring, holes);
        return polygon;
   }

    /**
     * 点面包含关系判断
     * 用途：判断一个面是否包含一个点，即一个点是否在一个面内
     * @param pointLocal 坐标点
     * @param outLines 轮廓线
     * @return
     */
   public static Boolean isPointInPoly(ObjectNode pointLocal, List<ObjectNode> outLines){
       //创建点
       Point point = createPoint(pointLocal);
       //创建面
       Polygon polygon = createPolygon(outLines);
       //判断点面包含关系
       boolean flag = polygon.contains(point);
       return flag;
   }

}
